pub trait Number: Clone + 'static { }Expand description
Marker trait for possible literals.
This trait is somewhat of a crutch, necessary to ensure that function wrappers can accept number arguments and distinguish them from other types (booleans, vectors, tuples, etc.).
Dyn Compatibility
This trait is not dyn compatible.
In older versions of Rust, dyn compatibility was called "object safety".
Implementations on Foreign Types
impl Number for BigInt
Available on crate feature
num-bigint only.impl Number for BigUint
Available on crate feature
num-bigint only.impl Number for Complex32
Available on crate feature
num-complex only.impl Number for Complex64
Available on crate feature
num-complex only.